package dev.kske.chess.uci;

import java.util.ArrayList;
import java.util.List;

import dev.kske.chess.board.Move;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>UCIInfo.java</strong><br>
 * Created: <strong>28.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class UCIInfo {

	private int			depth, seldepth, time, nodes, multipv, currmovenumber, hashfull, nps, tbhits, sbhits, cpuload,
			cpunr;
	private List<Move>	pv, refutation, currline;
	private Move		currmove;
	private Score		score;
	private String		displayString;

	public UCIInfo(String line) {
		pv			= new ArrayList<>();
		refutation	= new ArrayList<>();
		currline	= new ArrayList<>();
		String[] tokens = line.split(" ");

		for (int i = 0; i < tokens.length; i++)
			switch (tokens[i]) {
				// Single parameter info
				case "depth":
				case "seldepth":
				case "time":
				case "nodes":
				case "multipv":
				case "currmove":
				case "currmovenumber":
				case "hashfull":
				case "nps":
				case "tbhits":
				case "sbhits":
				case "cpuload":
				case "string":
					try {
						getClass().getField(tokens[i]).set(this, tokens[++i]);
					} catch (Exception ex) {}
					break;
				// TODO: pv
				// TODO: refutation
				// TODO: currline
				case "pv":
					// TODO: parse infos after score
				case "score":
					score = new Score(line.substring(tokens[i].length() + 1));
				case "refutation":
				case "currline":
					break;
				default:
					System.err.printf("Unknown parameter '%s' for command 'info' found!%n", tokens[i]);
			}
	}

	public int getDepth() { return depth; }

	public int getSeldepth() { return seldepth; }

	public int getTime() { return time; }

	public int getNodes() { return nodes; }

	public int getMultipv() { return multipv; }

	public int getCurrmovenumber() { return currmovenumber; }

	public int getHashfull() { return hashfull; }

	public int getNps() { return nps; }

	public int getTbhits() { return tbhits; }

	public int getSbhits() { return sbhits; }

	public int getCpuload() { return cpuload; }

	public int getCpunr() { return cpunr; }

	public List<Move> getPv() { return pv; }

	public List<Move> getRefutation() { return refutation; }

	public List<Move> getCurrline() { return currline; }

	public Move getCurrmove() { return currmove; }

	public Score getScore() { return score; }

	public String getDisplayString() { return displayString; }

	public static class Score {

		private int		cp, mate;
		private boolean	lowerbound, upperbound;

		public Score(String line) {
			String[] tokens = line.split(" ");
			switch (tokens[0]) {
				case "cp":
					cp = Integer.parseInt(tokens[1]);
					break;
				case "mate":
					mate = Integer.parseInt(tokens[1]);
					break;
				case "lowerbound":
					lowerbound = true;
					break;
				case "upperbound":
					upperbound = true;
					break;
				default:
					System.err.printf("Unknown parameter '%s' for command 'score' found!%n", tokens[0]);
			}
		}

		public int getCp() { return cp; }

		public int getMate() { return mate; }

		public boolean isLowerbound() { return lowerbound; }

		public boolean isUpperbound() { return upperbound; }
	}
}
